/*
jGuard is a security framework based on top of jaas (java authentication and authorization security).
it is written for web applications, to resolve simply, access control problems.
 
http://sourceforge.net/projects/jguard/

Copyright (C) 2004  Charles GAY

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


jGuard project home page:
http://sourceforge.net/projects/jguard/

*/
package net.sf.jguard.jee.listeners;

import javax.servlet.http.HttpSession;
import javax.servlet.http.HttpSessionEvent;
import javax.servlet.http.HttpSessionListener;

import net.sf.jguard.core.CoreConstants;
import net.sf.jguard.core.authentication.AuthenticationUtils;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Listener called when the Session is created or destroyed.
 * @author <a href="mailto:diabolo512@users.sourceforge.net">Charles Gay</a>
 */
public class SessionListener implements HttpSessionListener {

	
	static public final Logger logger = LoggerFactory.getLogger(SessionListener.class);
	
    /** 
     * constructor.
     */
    public SessionListener() {
        super();
    }

    /**
     * method called when the sesion is created.
     * @see javax.servlet.http.HttpSessionListener#sessionCreated(javax.servlet.http.HttpSessionEvent)
     */
    public void sessionCreated(HttpSessionEvent httpSessionEvent) {

    }

    /**
     * method called when session expired: we logout the loginModules.
     * The subject and all others objects bound to the Session will be removed.
     * @see javax.servlet.http.HttpSessionListener#sessionDestroyed(javax.servlet.http.HttpSessionEvent)
     */
    public void sessionDestroyed(HttpSessionEvent httpSessionEvent) {
       HttpSession session = httpSessionEvent.getSession();
       if(session!= null){
    	   try{
		       AuthenticationUtils authUtils = (AuthenticationUtils)session.getAttribute(CoreConstants.AUTHN_UTILS);
		       if(authUtils!=null){
		           authUtils.logout();  
		       }
		       session.removeAttribute(CoreConstants.AUTHN_UTILS);
    	   }catch(IllegalStateException ise){
    		   logger.debug(" session is already invalidated ");
    	   }
       }
       
       

    }

}
